@@ -174,7 +174,8 @@ ecb_caldav_connect_sync (ECalMetaBackend *meta_backend,
 		gboolean credentials_empty;
 		gboolean is_ssl_error;
 
-		credentials_empty = !credentials || !e_named_parameters_count (credentials);
+		credentials_empty = (!credentials || !e_named_parameters_count (credentials)) &&
+			e_soup_session_get_authentication_requires_credentials (E_SOUP_SESSION (cbdav->priv->webdav));
 		is_ssl_error = g_error_matches (local_error, SOUP_HTTP_ERROR, SOUP_STATUS_SSL_FAILED);
 
 		*out_auth_result = E_SOURCE_AUTHENTICATION_ERROR;
@@ -183,22 +184,25 @@ ecb_caldav_connect_sync (ECalMetaBackend *meta_backend,
 		if (g_error_matches (local_error, SOUP_HTTP_ERROR, SOUP_STATUS_CANCELLED)) {
 			local_error->domain = G_IO_ERROR;
 			local_error->code = G_IO_ERROR_CANCELLED;
-			g_propagate_error (error, local_error);
 		} else if (g_error_matches (local_error, SOUP_HTTP_ERROR, SOUP_STATUS_FORBIDDEN) && credentials_empty) {
 			*out_auth_result = E_SOURCE_AUTHENTICATION_REQUIRED;
-			g_clear_error (&local_error);
 		} else if (g_error_matches (local_error, SOUP_HTTP_ERROR, SOUP_STATUS_UNAUTHORIZED)) {
 			if (credentials_empty)
 				*out_auth_result = E_SOURCE_AUTHENTICATION_REQUIRED;
 			else
 				*out_auth_result = E_SOURCE_AUTHENTICATION_REJECTED;
-			g_clear_error (&local_error);
-		} else if (local_error) {
+		} else if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CONNECTION_REFUSED) ||
+			   (!e_soup_session_get_authentication_requires_credentials (E_SOUP_SESSION (cbdav->priv->webdav)) &&
+			   g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))) {
+			*out_auth_result = E_SOURCE_AUTHENTICATION_REJECTED;
+		} else if (!local_error) {
+			g_set_error_literal (&local_error, G_IO_ERROR, G_IO_ERROR_FAILED,
+				_("Unknown error"));
+		}
+
+		if (local_error) {
 			g_propagate_error (error, local_error);
 			local_error = NULL;
-		} else {
-			g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-				_("Unknown error"));
 		}
 
 		if (is_ssl_error) {
